import asyncio
from nicegui import app, ui
import sys
import os
sys.path.append(os.getcwd())
from src.nicegui_super_extensions.reconnect_box import set_reconnect_box
from src.nicegui_super_extensions.markdown import (
    MarkdownPreViewer,
    initMarkdownPreViewerDependence,
)

markdown = r"""
## 教程

这是一篇讲解如何正确使用 **Markdown** 的排版示例，学会这个很有必要，能让你的文章有更佳清晰的排版。

> 引用文本：Markdown is a text formatting syntax inspired

## 语法指导

### 普通内容

这段内容展示了在内容里面一些排版格式，比如：

- **加粗** - `**加粗**`
- *倾斜* - `*倾斜*`
- ~~删除线~~ - `~~删除线~~`
- `Code 标记` - `` `Code 标记` ``
- [超级链接](https://ld246.com) - `[超级链接](https://ld246.com)`
- [username@gmail.com](mailto:username@gmail.com) - `[username@gmail.com](mailto:username@gmail.com)`

### 提及用户

@Vanessa 通过 `@User` 可以在内容中提及用户，被提及的用户将会收到系统通知。

> NOTE:
>
> 1. @用户名之后需要有一个空格
> 2. 新手没有艾特的功能权限

### 表情符号 Emoji

支持大部分标准的表情符号，可使用输入法直接输入，也可手动输入字符格式。通过输入 `:` 触发自动完成，可在个人设置中[设置常用表情](https://ld246.com/settings/function)。

#### 一些表情例子

:smile: :laughing: :dizzy_face: :sob: :cold_sweat: :sweat_smile:  :cry: :triumph: :heart_eyes: :relieved:
:+1: :-1: :100: :clap: :bell: :gift: :question: :bomb: :heart: :coffee: :cyclone: :bow: :kiss: :pray: :anger:

### 大标题 - Heading 3

你可以选择使用 H1 至 H6，使用 ##(N) 打头。建议帖子或回帖中的顶级标题使用 Heading 3，不要使用 1 或 2，因为 1 是系统站点级，2 是帖子标题级。

> NOTE: 别忘了 # 后面需要有空格！

#### Heading 4

##### Heading 5

###### Heading 6

### 图片

```
![alt 文本](http://image-path.png)
![alt 文本](http://image-path.png "图片 Title 值")
```

支持复制粘贴直接上传。

### 代码块

#### 普通

```
*emphasize*    **strong**
_emphasize_    __strong__
var a = 1
```

#### 语法高亮支持

如果在 ``` 后面跟随语言名称，可以有语法高亮的效果哦，比如:

##### 演示 Go 代码高亮

```go
package main

import "fmt"

func main() {
	fmt.Println("Hello, 世界")
}
```

##### 演示 Java 高亮

```java
public class HelloWorld {

    public static void main(String[] args) {
        System.out.println("Hello World!");
    }

}
```

> Tip: 语言名称支持下面这些: `ruby`, `python`, `js`, `html`, `erb`, `css`, `coffee`, `bash`, `json`, `yml`, `xml` ...

### 有序、无序、任务列表

#### 无序列表

- Java
  - Spring
    - IoC
    - AOP
- Go
  - gofmt
  - Wide
- Node.js
  - Koa
  - Express

#### 有序列表

1. Node.js
   1. Express
   2. Koa
   3. Sails
2. Go
   1. gofmt
   2. Wide
3. Java
   1. Latke
   2. IDEA

#### 任务列表

- [x] 发布 Sym
- [X] 发布 Solo
- [ ] 预约牙医

### 表格

如果需要展示数据什么的，可以选择使用表格。

| header 1 | header 2 |
| -------- | -------- |
| cell 1   | cell 2   |
| cell 3   | cell 4   |
| cell 5   | cell 6   |

### 隐藏细节

<details>
<summary>这里是摘要部分。</summary>
这里是细节部分。
</details>

### 段落

空行可以将内容进行分段，便于阅读。（这是第一段）

使用空行在 Markdown 排版中相当重要。（这是第二段）

### 链接引用

[链接文本][链接标识]

[链接标识]: https://b3log.org

```
[链接文本][链接标识]

[链接标识]: https://b3log.org
```

### 数学公式

多行公式块：

$$
\frac{1}{
  \Bigl(\sqrt{\phi \sqrt{5}}-\phi\Bigr) e^{
  \frac25 \pi}} = 1+\frac{e^{-2\pi}} {1+\frac{e^{-4\pi}} {
    1+\frac{e^{-6\pi}}
    {1+\frac{e^{-8\pi}}{1+\cdots}}
  }
}
$$

行内公式：

公式 $a^2 + b^2 = \color{red}c^2$ 是行内。

### 脑图
```mindmap
- 教程
- 语法指导
  - 普通内容
  - 提及用户
  - 表情符号 Emoji
    - 一些表情例子
  - 大标题 - Heading 3
    - Heading 4
      - Heading 5
        - Heading 6
  - 图片
  - 代码块
    - 普通
    - 语法高亮支持
      - 演示 Go 代码高亮
      - 演示 Java 高亮
  - 有序、无序、任务列表
    - 无序列表
    - 有序列表
    - 任务列表
  - 表格
  - 隐藏细节
  - 段落
  - 链接引用
  - 数学公式
  - 脑图
  - 流程图
  - 时序图
  - 甘特图
  - 图表
  - 五线谱
  - Graphviz
  - 多媒体
  - 脚注
- 快捷键
```

### 流程图

```mermaid
graph TB
    c1-->a2
    subgraph one
    a1-->a2
    end
    subgraph two
    b1-->b2
    end
    subgraph three
    c1-->c2
    end
```

### 时序图

```mermaid
sequenceDiagram
    Alice->>John: Hello John, how are you?
    loop Every minute
        John-->>Alice: Great!
    end
```

### 甘特图

```mermaid
gantt
    title A Gantt Diagram
    dateFormat  YYYY-MM-DD
    section Section
    A task           :a1, 2019-01-01, 30d
    Another task     :after a1  , 20d
    section Another
    Task in sec      :2019-01-12  , 12d
    another task      : 24d
```

### 图表

```echarts
{
  "title": { "text": "最近 30 天" },
  "tooltip": { "trigger": "axis", "axisPointer": { "lineStyle": { "width": 0 } } },
  "legend": { "data": ["帖子", "用户", "回帖"] },
  "xAxis": [{
      "type": "category",
      "boundaryGap": false,
      "data": ["2019-05-08","2019-05-09","2019-05-10","2019-05-11","2019-05-12","2019-05-13","2019-05-14","2019-05-15","2019-05-16","2019-05-17","2019-05-18","2019-05-19","2019-05-20","2019-05-21","2019-05-22","2019-05-23","2019-05-24","2019-05-25","2019-05-26","2019-05-27","2019-05-28","2019-05-29","2019-05-30","2019-05-31","2019-06-01","2019-06-02","2019-06-03","2019-06-04","2019-06-05","2019-06-06","2019-06-07"],
      "axisTick": { "show": false },
      "axisLine": { "show": false }
  }],
  "yAxis": [{ "type": "value", "axisTick": { "show": false }, "axisLine": { "show": false }, "splitLine": { "lineStyle": { "color": "rgba(0, 0, 0, .38)", "type": "dashed" } } }],
  "series": [
    {
      "name": "帖子", "type": "line", "smooth": true, "itemStyle": { "color": "#d23f31" }, "areaStyle": { "normal": {} }, "z": 3,
      "data": ["18","14","22","9","7","18","10","12","13","16","6","9","15","15","12","15","8","14","9","10","29","22","14","22","9","10","15","9","9","15","0"]
    },
    {
      "name": "用户", "type": "line", "smooth": true, "itemStyle": { "color": "#f1e05a" }, "areaStyle": { "normal": {} }, "z": 2,
      "data": ["31","33","30","23","16","29","23","37","41","29","16","13","39","23","38","136","89","35","22","50","57","47","36","59","14","23","46","44","51","43","0"]
    },
    {
      "name": "回帖", "type": "line", "smooth": true, "itemStyle": { "color": "#4285f4" }, "areaStyle": { "normal": {} }, "z": 1,
      "data": ["35","42","73","15","43","58","55","35","46","87","36","15","44","76","130","73","50","20","21","54","48","73","60","89","26","27","70","63","55","37","0"]
    }
  ]
}
```

### 五线谱

```abc
X: 24
T: Clouds Thicken
C: Paul Rosen
S: Copyright 2005, Paul Rosen
M: 6/8
L: 1/8
Q: 3/8=116
R: Creepy Jig
K: Em
|:"Em"EEE E2G|"C7"_B2A G2F|"Em"EEE E2G|\
"C7"_B2A "B7"=B3|"Em"EEE E2G|
"C7"_B2A G2F|"Em"GFE "D (Bm7)"F2D|\
1"Em"E3-E3:|2"Em"E3-E2B|:"Em"e2e gfe|
"G"g2ab3|"Em"gfeg2e|"D"fedB2A|"Em"e2e gfe|\
"G"g2ab3|"Em"gfe"D"f2d|"Em"e3-e3:|
```

### Graphviz

```graphviz
digraph finite_state_machine {
    rankdir=LR;
    size="8,5"
    node [shape = doublecircle]; S;
    node [shape = point ]; qi

    node [shape = circle];
    qi -> S;
    S  -> q1 [ label = "a" ];
    S  -> S  [ label = "a" ];
    q1 -> S  [ label = "a" ];
    q1 -> q2 [ label = "ddb" ];
    q2 -> q1 [ label = "b" ];
    q2 -> q2 [ label = "b" ];
}
```

### 多媒体

支持 v.qq.com，youtube.com，youku.com，coub.com，facebook.com/video，dailymotion.com，.mp4，.m4v，.ogg，.ogv，.webm，.mp3，.wav 链接解析

https://v.qq.com/x/cover/zf2z0xpqcculhcz/y0016tj0qvh.html

### 脚注

这里是一个脚注引用[^1]，这里是另一个脚注引用[^bignote]。

[^1]: 第一个脚注定义。
[^bignote]: 脚注定义可使用多段内容。

    缩进对齐的段落包含在这个脚注定义内。

    ```
    可以使用代码块。
    ```

    还有其他行级排版语法，比如**加粗**和[链接](https://b3log.org)。

```
这里是一个脚注引用[^1]，这里是另一个脚注引用[^bignote]。
[^1]: 第一个脚注定义。
[^bignote]: 脚注定义可使用多段内容。

    缩进对齐的段落包含在这个脚注定义内。

    ```
    可以使用代码块。
    ```

    还有其他行级排版语法，比如**加粗**和[链接](https://b3log.org)。


```

## 快捷键

我们的编辑器支持很多快捷键，具体请参考 [键盘快捷键](https://ld246.com/article/1582778815353)

## Multilingual
中文
English
한국어
日本語
Français
"""


async def show_in_dialog():
    """click to update"""

    with ui.dialog() as dialog, ui.card().classes("w-full").style("max-width: none"):
        # test use with other component
        md = MarkdownPreViewer()
    dialog.open()
    md.render(markdown, anchor=1)


async def show_in_dialog_and_update():
    """click to update"""
    with ui.dialog() as dialog, ui.card().classes("w-full").style("max-width: none"):
        # test use with other component
        md = MarkdownPreViewer()
    dialog.open()
    md.render(markdown, anchor=1)
    await asyncio.sleep(2)
    md.render("# 测试第二次发送")


async def show():
    """click to update"""
    md = MarkdownPreViewer().classes("w-96")
    md.render(markdown, anchor=1)


async def show_and_update():
    """click to update"""
    md = MarkdownPreViewer()
    md.render(markdown, anchor=0)
    await asyncio.sleep(2)
    md.render("# 测试第二次发送")


initMarkdownPreViewerDependence()
button = ui.button("show in dialog", on_click=show_in_dialog)
button = ui.button("show", on_click=show)
button = ui.button("show in dialog and update", on_click=show_in_dialog_and_update)
button = ui.button("show and update", on_click=show_and_update)
app.on_connect(lambda: set_reconnect_box())

ui.run(reload=False, dark=None)



from __future__ import annotations

import random

from nicegui import ui


class ReorderableItem(ui.card):
    """
    Represents a reorderable item that can be moved within a `ReorderableRow` or `ReorderableColumn`.
    """

    def __init__(self, enable_dragging: bool = True, drop_check_duration=3) -> None:
        """
        Initializes a `ReorderableItem`.

        Args:
            enable_dragging: Whether the item is reorderable. Defaults to True.
        """
        super().__init__()
        self.drag_enabled = enable_dragging
        self.drop_check_duration = drop_check_duration

        # Create the card with reorderable props and style
        with self.props("draggable").classes(f"cursor-pointer").style(
            "box-shadow: none;"
        ):
            self.build()

        # Set up event listeners for drag and drop
        self.on("dragstart", self.on_dragstart)
        self.on("dragenter", self.on_dragenter)
        self.on("dragleave", self.on_dragleave)
        self.on("dragover.prevent", self.on_dragover_prevent)
        self.on("drop", self.on_drop)

        # Disable dragging if not enabled
        if not self.drag_enabled:
            self.disable_drag()

    def disable_drag(self) -> None:
        """Disables dragging for this item."""
        self.drag_enabled = False
        self.classes(remove="cursor-pointer")
        self.props(remove="reorderable")

    def enable_drag(self) -> None:
        """Enables dragging for this item."""
        self.drag_enabled = True
        self.classes(add="cursor-pointer")
        self.props(add="reorderable")

    def get_parent(self) -> ReorderableColumn | ReorderableRow | None:
        """Returns the parent container (ReorderableColumn or ReorderableRow) of this item."""
        return self.parent_slot.parent  # type:ignore

    def delete_from_parent(self) -> None:
        """Removes this item from its parent container."""
        parent = self.get_parent()
        if parent is not None:
            parent.remove(self)

    def build(self) -> None:
        """
        Override this method to build the content of the reorderable card.
        """

    def on_dragstart(self) -> None:
        """
        Event handler for the 'dragstart' event.
        Highlights the dragged item and sets it as the currently dragged item.
        """
        if not self.drag_enabled:
            return
        global _dragged
        _dragged = self

    def on_drop_check(self) -> None:
        """
        Event handler for checking if the dragged item should be dropped.
        Removes the highlight from the dragged item and cancels the timer.
        """
        global _dragged
        if _dragged is not None:
            self.timer.cancel()

    def on_dragenter(self) -> None:
        """
        Event handler for the 'dragenter' event.
        Highlights the target item and sets a timer to check for dropping.
        """
        if not self.drag_enabled:
            return

        self.timer = ui.timer(self.drop_check_duration, self.on_drop_check)

    def on_dragleave(self) -> None:
        """
        Event handler for the 'dragleave' event.
        Removes the highlight from the target item and cancels the timer.
        """
        if not self.drag_enabled:
            return

        if hasattr(self, "timer"):
            self.timer.cancel()

    def on_drop(self) -> None:
        """
        Event handler for the 'drop' event.
        Moves the dragged item to the target position and resets the dragged item.
        """
        if not self.drag_enabled:
            return
        global _dragged
        if not _dragged:
            return

        parent = self.get_parent()
        if parent is None:
            return

        children = parent.get_reorderable_children()
        try:
            self_index = children.index(self)
        except ValueError:
            return

        _dragged.move(target_index=self_index)
        self.on_dragleave()

    def on_dragover_prevent(self):
        """Prevent default dragover event to allow drop event"""
        return


class ReorderableRow(ui.row):
    """
    Represents a row that can contain reorderable items.
    """

    def get_reorderable_children(self) -> list[ReorderableItem]:
        """
        Returns a list of reorderable items in this row.
        """
        children = []
        for i in self.default_slot.children:
            if issubclass(i.__class__, ReorderableItem):
                if i.drag_enabled:
                    children.append(i)
        return children

    def shuffle(self) -> None:
        """
        Shuffles the order of reorderable items in this row.
        """
        children = self.get_reorderable_children()
        random.shuffle(children)
        for i in children:
            i.move(target_index=0)


class ReorderableColumn(ui.column):
    """
    Represents a column that can contain reorderable items.
    """

    def get_reorderable_children(self) -> list[ReorderableItem]:
        """
        Returns a list of reorderable items in this column.
        """
        children = []
        for i in self.default_slot.children:
            if issubclass(i.__class__, ReorderableItem):
                if i.drag_enabled:
                    children.append(i)
        return children

    def shuffle(self) -> None:
        """
        Shuffles the order of reorderable items in this column.
        """
        children = self.get_reorderable_children()
        random.shuffle(children)
        for i in children:
            i.move(target_index=0)



from fastapi import FastAPI
from nicegui import ui
import sys
import os

sys.path.append(os.getcwd())
from src.nicegui_super_extensions.reorderable import (
    ReorderableItem,
    ReorderableColumn,
)

app = FastAPI()


def init(fastapi_app: FastAPI) -> None:

    @ui.page("/")
    def show():
        with ReorderableColumn() as col:
            with ReorderableItem() as draggable:
                ui.label("Draggable Item 1")

            with ReorderableItem() as draggable:
                ui.label("Draggable Item 2")

            with ReorderableItem() as draggable:
                ui.label("Draggable Item 3")

    ui.run_with(
        fastapi_app, mount_path="/", storage_secret="pick your private secret here"
    )


if __name__ == "__main__":
    import uvicorn

    init(app)
    uvicorn.run(app, host="127.0.0.1", port=8000)



from nicegui import ui


def set_reconnect_box(title="与服务器连接断开👽", message="正在重连中。。。"):
    """change the connect_box text, must call after ui is running"""
    ui.run_javascript(
        'const popup = document.getElementById("popup");\n'
        + "if (popup) {\n"
        + 'const spans = popup.querySelectorAll("span");\n'
        + 'spans[0].textContent ="'
        + title
        + '";\n'
        + 'spans[1].textContent ="'
        + message
        + '";\n}'
    )
